//************************************************************************
//* EloSoft components for Delphi 6 (copyright 2003)                     *
//************************************************************************

unit SkinBitBtn;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Buttons, Graphics,
  CommCtrl, Forms, Dialogs, StdCtrls;

type
  TSkinBitBtn = class(TBitBtn)
  private
    { Private declarations }
    DisabledBitmap, GrayBitmap: TBitmap;
    FFocused: Boolean;
    FCanvas: TCanvas;
    FGrayInactive: Boolean;
    FGlyphTransparent: Boolean;
    FDown: Boolean;
    FFlat: Boolean;
    FBtnAutosize: Boolean;
    FKind: TBitBtnKind;
    FMarkBtnFocused: Boolean;

    FSkinUp: TBitmap;
    FSkinDown: TBitmap;
    FSkinDisabled: TBitmap;
    FSkinDefault: TBitmap;
    FSkinFocused: TBitmap;
    FSkinFlat: TBitmap;

    procedure SetGrayInactive(Value: Boolean);
    procedure SetGlyphTransparent(Value: Boolean);
    procedure SetDown(Value: Boolean);
    procedure SetFlat(Value: Boolean);
    procedure SetBtnAutosize(Value: Boolean);
    function GetKind: TBitBtnKind;
    procedure SetKind(Value: TBitBtnKind);
    procedure SetMarkBtnFocused(Value: Boolean);

    procedure SetSkinUp(Value: TBitmap);
    procedure SetSkinDown(Value: TBitmap);
    procedure SetSkinDisabled(Value: TBitmap);
    procedure SetSkinDefault(Value: TBitmap);
    procedure SetSkinFocused(Value: TBitmap);
    procedure SetSkinFlat(Value: TBitmap);
  protected
    { Protected declarations }
    procedure ClearSkinBitBtn(Canvas: TCanvas; Rect: TRect);
    procedure DrawItem(const DrawItemStruct: TDrawItemStruct);
    function Draw(Canvas: TCanvas; const Client: TRect;
      const Offset: TPoint; const Caption: string; Layout: TButtonLayout;
      Margin, Spacing: Integer; State: TButtonState; BiDiFlags: LongInt): TRect;
    procedure CalcLayout(Canvas: TCanvas; const Client: TRect;
      const Offset: TPoint; const Caption: string; Layout: TButtonLayout; Margin,
      Spacing: Integer; var GlyphPos: TPoint; var TextBounds: TRect;
      BiDiFlags: LongInt);
    function MapColor(Color: TColor): TColor;
    procedure DrawButtonGlyph(Canvas: TCanvas; CurBitmap: TBitmap;
      const GlyphPos: TPoint; State: TButtonState; Transparent: Boolean;
      Stretch: Boolean; ClyphRect: TRect);
    procedure DrawButtonText(Canvas: TCanvas; const Caption: string;
      TextBounds: TRect; State: TButtonState; BiDiFlags: LongInt);

    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure CMMouseLeave( var Message: TMessage ); Message CM_MOUSELEAVE;
    procedure CMMouseEnter( var Message: TMessage ); Message CM_MOUSEENTER;
    procedure CMFocusChanged(var Message: TCMFocusChanged); message CM_FOCUSCHANGED;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    { Published declarations }
    property GrayInactive: Boolean read FGrayInactive write SetGrayInactive;
    property GlyphTransparent: Boolean read FGlyphTransparent write SetGlyphTransparent;
    property Down: Boolean read FDown write SetDown;
    property Flat: Boolean read FFlat write SetFlat;
    property BtnAutosize: Boolean read FBtnAutosize write SetBtnAutosize;
    property Kind: TBitBtnKind read GetKind write SetKind default bkCustom;
    property MarkBtnFocused: Boolean read FMarkBtnFocused write SetMarkBtnFocused;

    property SkinUp: TBitmap read FSkinUp write SetSkinUp;
    property SkinDown: TBitmap read FSkinDown write SetSkinDown;
    property SkinDisabled: TBitmap read FSkinDisabled write SetSkinDisabled;
    property SkinDefault: TBitmap read FSkinDefault write SetSkinDefault;
    property SkinFocused: TBitmap read FSkinFocused write SetSkinFocused;
    property SkinFlat: TBitmap read FSkinFlat write SetSkinFlat;
  end;

procedure Register;

resourcestring
  SOKButton = 'OK';
  SCancelButton = 'Cancel';
  SYesButton = '&Yes';
  SNoButton = '&No';
  SHelpButton = '&Help';
  SCloseButton = '&Close';
  SIgnoreButton = '&Ignore';
  SRetryButton = '&Retry';
  SAbortButton = 'Abort';
  SAllButton = '&All';

implementation

{$R SkinBitBtnKind.RES}

{ TSkinBitBtn data }
var
  SkinBitBtnResNames: array[TBitBtnKind] of PChar = (
    nil, 'SBOK', 'SBCANCEL', 'SBHELP', 'SBYES', 'SBNO',
    'SBCLOSE', 'SBABORT', 'SBRETRY', 'SBIGNORE', 'SBALL');
  SkinBitBtnCaptions: array[TBitBtnKind] of Pointer = (
    nil, @SOKButton, @SCancelButton, @SHelpButton, @SYesButton, @SNoButton,
    @SCloseButton, @SAbortButton, @SRetryButton, @SIgnoreButton,
    @SAllButton);
  SkinBitBtnModalResults: array[TBitBtnKind] of TModalResult = (
    0, mrOk, mrCancel, 0, mrYes, mrNo, 0, mrAbort, mrRetry, mrIgnore,
    mrAll);
  SkinBitBtnGlyphs: array[TBitBtnKind] of TBitmap;

function GetBitBtnGlyph(Kind: TBitBtnKind): TBitmap;
begin
  if Kind= bkCustom then
  begin
   Result:= nil;
   exit;
  end;

  if SkinBitBtnGlyphs[Kind] = nil then
  begin
    SkinBitBtnGlyphs[Kind] := TBitmap.Create;
    SkinBitBtnGlyphs[Kind].LoadFromResourceName(HInstance, SkinBitBtnResNames[Kind]);
  end;
  Result := SkinBitBtnGlyphs[Kind];
end;

function HeightOf(R: TRect): Integer;
begin
  Result := R.Bottom - R.Top;
end;

function WidthOf(R: TRect): Integer;
begin
  Result := R.Right - R.Left;
end;

function CreateDisabledBitmapEx(FOriginal: TBitmap; OutlineColor, BackColor,
  HighlightColor, ShadowColor: TColor; DrawHighlight: Boolean): TBitmap;
var
  MonoBmp: TBitmap;
  IRect: TRect;
begin
  IRect := Rect(0, 0, FOriginal.Width, FOriginal.Height);
  Result := TBitmap.Create;
  try
    Result.Width := FOriginal.Width;
    Result.Height := FOriginal.Height;
    MonoBmp := TBitmap.Create;
    try
      with MonoBmp do begin
        Width := FOriginal.Width;
        Height := FOriginal.Height;
        Canvas.CopyRect(IRect, FOriginal.Canvas, IRect);
{$IFDEF RX_D3}
        HandleType := bmDDB;
{$ENDIF}
        Canvas.Brush.Color := OutlineColor;
        if Monochrome then begin
          Canvas.Font.Color := clWhite;
          Monochrome := False;
          Canvas.Brush.Color := clWhite;
        end;
        Monochrome := True;
      end;
      with Result.Canvas do begin
        Brush.Color := BackColor;
        FillRect(IRect);
        if DrawHighlight then begin
          Brush.Color := HighlightColor;
          SetTextColor(Handle, clBlack);
          SetBkColor(Handle, clWhite);
          BitBlt(Handle, 1, 1, WidthOf(IRect), HeightOf(IRect),
            MonoBmp.Canvas.Handle, 0, 0, $00E20746);
        end;
        Brush.Color := ShadowColor;
        SetTextColor(Handle, clBlack);
        SetBkColor(Handle, clWhite);
        BitBlt(Handle, 0, 0, WidthOf(IRect), HeightOf(IRect),
          MonoBmp.Canvas.Handle, 0, 0, $00E20746);
      end;
    finally
      MonoBmp.Free;
    end;
  except
    Result.Free;
    raise;
  end;
end;

function CreateDisabledBitmap(FOriginal: TBitmap; OutlineColor: TColor): TBitmap;
begin
  Result := CreateDisabledBitmapEx(FOriginal, OutlineColor,
    clBtnFace, clBtnHighlight, clBtnShadow, True);
end;

procedure Register;
begin
  RegisterComponents('EloSoft', [TSkinBitBtn]);
end;

constructor TSkinBitBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCanvas := TCanvas.Create;
  GrayBitmap:= TBitmap.Create;
  DisabledBitmap:= TBitmap.Create;
  FFocused:= False;
  FKind := bkCustom;
  FGrayInactive:= True;
  FGlyphTransparent:= True;
  FDown:= False;
  FFlat:= False;
  FBtnAutosize:= False;
  FMarkBtnFocused:= True;

  FSkinUp:= TBitmap.Create;
  FSkinDown:= TBitmap.Create;
  FSkinDisabled:= TBitmap.Create;
  FSkinDefault:= TBitmap.Create;
  FSkinFocused:= TBitmap.Create;
  FSkinFlat:= TBitmap.Create;
end;

destructor TSkinBitBtn.Destroy;
begin
  if GrayBitmap<> nil then GrayBitmap.Free;
  GrayBitmap:= nil;
  if DisabledBitmap<> nil then DisabledBitmap.Free;
  DisabledBitmap:= nil;

  if FSkinUp<> nil then FSkinUp.Free;
  FSkinUp:= nil;
  if FSkinDown<> nil then FSkinDown.Free;
  FSkinDown:= nil;
  if FSkinDisabled<> nil then FSkinDisabled.Free;
  FSkinDisabled:= nil;
  if FSkinDefault<> nil then FSkinDefault.Free;
  FSkinDefault:= nil;
  if FSkinFocused<> nil then FSkinFocused.Free;
  FSkinFocused:= nil;
  if FSkinFlat<> nil then FSkinFlat.Free;
  FSkinFlat:= nil;

  FCanvas.Free;
  inherited Destroy;
end;

procedure TSkinBitBtn.Click;
var
  Form: TCustomForm;
  Control: TWinControl;
begin
  Form := GetParentForm(Self);
  if Form <> nil then
  begin
   if Form.ClassName= 'TMessageForm' then Form.ModalResult := ModalResult
   else
   begin
    case FKind of
     bkClose: Form.Close;
     bkHelp:  begin
               Control := Self;
               while (Control <> nil) and (Control.HelpContext = 0) do
                     Control := Control.Parent;
               if Control <> nil then Application.HelpContext(Control.HelpContext)
              end;
     else Form.ModalResult := ModalResult;
    end; // end Case FKind
   end; // end else if Form.ClassName= 'TMessageForm'
  end; // end if Form <> nil

  if Assigned(OnClick) and (Action <> nil)
  and (@OnClick <> @Action.OnExecute) then OnClick(Self)
  else if not (csDesigning in ComponentState)
       and (ActionLink <> nil) then ActionLink.Execute(Self)
           else if Assigned(OnClick) then OnClick(Self);
end;

procedure TSkinBitBtn.CMMouseLeave( Var Message: TMessage );
begin
  inherited;
  FFocused:=False;
  invalidate;
end;

procedure TSkinBitBtn.CMMouseEnter( Var Message: TMessage );
begin
  inherited;
  FFocused:=True;
  invalidate;
end;

procedure TSkinBitBtn.CMFocusChanged(var Message: TCMFocusChanged);
begin
  inherited;
  Invalidate;
end;

function TSkinBitBtn.Draw(Canvas: TCanvas; const Client: TRect;
  const Offset: TPoint; const Caption: string; Layout: TButtonLayout;
  Margin, Spacing: Integer; State: TButtonState; BiDiFlags: LongInt): TRect;
var
  GlyphPos: TPoint;
begin
  CalcLayout(Canvas, Client, Offset, Caption, Layout, Margin, Spacing,
    GlyphPos, Result, BiDiFlags);
  DrawButtonGlyph(Canvas, Glyph, GlyphPos, State, FGlyphTransparent, False, Client);
  DrawButtonText(Canvas, Caption, Result, State, BiDiFlags);
end;

procedure TSkinBitBtn.CalcLayout(Canvas: TCanvas; const Client: TRect;
  const Offset: TPoint; const Caption: string; Layout: TButtonLayout; Margin,
  Spacing: Integer; var GlyphPos: TPoint; var TextBounds: TRect;
  BiDiFlags: LongInt);
var
  TextPos: TPoint;
  ClientSize, GlyphSize, TextSize: TPoint;
  TotalSize: TPoint;
begin
  if (BiDiFlags and DT_RIGHT) = DT_RIGHT then
    if Layout = blGlyphLeft then Layout := blGlyphRight
    else
      if Layout = blGlyphRight then Layout := blGlyphLeft;
  { calculate the item sizes }
  ClientSize := Point(Client.Right - Client.Left, Client.Bottom -
    Client.Top);

  if Glyph <> nil then
    GlyphSize := Point(Glyph.Width div NumGlyphs, Glyph.Height) else
    GlyphSize := Point(0, 0);
    
  if Length(Caption) > 0 then
  begin
    TextBounds := Rect(0, 0, Client.Right - Client.Left, 0);
    DrawText(Canvas.Handle, PChar(Caption), Length(Caption), TextBounds,
      DT_CALCRECT or BiDiFlags);
    TextSize := Point(TextBounds.Right - TextBounds.Left, TextBounds.Bottom -
      TextBounds.Top);
  end
  else
  begin
    TextBounds := Rect(0, 0, 0, 0);
    TextSize := Point(0,0);
  end;
    
  { If the layout has the glyph on the right or the left, then both the
    text and the glyph are centered vertically.  If the glyph is on the top
    or the bottom, then both the text and the glyph are centered horizontally.}
  if Layout in [blGlyphLeft, blGlyphRight] then
  begin
    GlyphPos.Y := (ClientSize.Y - GlyphSize.Y + 1) div 2;
    TextPos.Y := (ClientSize.Y - TextSize.Y + 1) div 2;
  end
  else
  begin
    GlyphPos.X := (ClientSize.X - GlyphSize.X + 1) div 2;
    TextPos.X := (ClientSize.X - TextSize.X + 1) div 2;
  end;

  { if there is no text or no bitmap, then Spacing is irrelevant }
  if (TextSize.X = 0) or (GlyphSize.X = 0) then
    Spacing := 0;

  { adjust Margin and Spacing }
  if Margin = -1 then
  begin
    if Spacing = -1 then
    begin
      TotalSize := Point(GlyphSize.X + TextSize.X, GlyphSize.Y + TextSize.Y);
      if Layout in [blGlyphLeft, blGlyphRight] then
        Margin := (ClientSize.X - TotalSize.X) div 3
      else
        Margin := (ClientSize.Y - TotalSize.Y) div 3;
      Spacing := Margin;
    end
    else
    begin
      TotalSize := Point(GlyphSize.X + Spacing + TextSize.X, GlyphSize.Y +
        Spacing + TextSize.Y);
      if Layout in [blGlyphLeft, blGlyphRight] then
        Margin := (ClientSize.X - TotalSize.X + 1) div 2
      else
        Margin := (ClientSize.Y - TotalSize.Y + 1) div 2;
    end;
  end
  else
  begin
    if Spacing = -1 then
    begin
      TotalSize := Point(ClientSize.X - (Margin + GlyphSize.X), ClientSize.Y -
        (Margin + GlyphSize.Y));
      if Layout in [blGlyphLeft, blGlyphRight] then
        Spacing := (TotalSize.X - TextSize.X) div 2
      else
        Spacing := (TotalSize.Y - TextSize.Y) div 2;
    end;
  end;
    
  case Layout of
    blGlyphLeft:
      begin
        GlyphPos.X := Margin;
        TextPos.X := GlyphPos.X + GlyphSize.X + Spacing;
      end;
    blGlyphRight:
      begin
        GlyphPos.X := ClientSize.X - Margin - GlyphSize.X;
        TextPos.X := GlyphPos.X - Spacing - TextSize.X;
      end;
    blGlyphTop:
      begin
        GlyphPos.Y := Margin;
        TextPos.Y := GlyphPos.Y + GlyphSize.Y + Spacing;
      end;
    blGlyphBottom:
      begin
        GlyphPos.Y := ClientSize.Y - Margin - GlyphSize.Y;
        TextPos.Y := GlyphPos.Y - Spacing - TextSize.Y;
      end;
  end;
    
  { fixup the result variables }
  with GlyphPos do
  begin
    Inc(X, Client.Left + Offset.X);
    Inc(Y, Client.Top + Offset.Y);
  end;
  OffsetRect(TextBounds, TextPos.X + Client.Left + Offset.X,
    TextPos.Y + Client.Top + Offset.X);
end;

function TSkinBitBtn.MapColor(Color: TColor): TColor;
var
  Index: Byte;
begin
  if (Color = Glyph.TransparentColor) or (ColorToRGB(Color) =
    ColorToRGB(clBtnFace)) then Result := Color
  else begin
    Color := ColorToRGB(Color);
    Index := Byte(Longint(Word(GetRValue(Color)) * 77 +
      Word(GetGValue(Color)) * 150 + Word(GetBValue(Color)) * 29) shr 8);
    Result := RGB(Index, Index, Index);
  end;
end;

procedure TSkinBitBtn.DrawButtonGlyph(Canvas: TCanvas; CurBitmap: TBitmap;
  const GlyphPos: TPoint; State: TButtonState; Transparent: Boolean;
  Stretch: Boolean; ClyphRect: TRect);
var
   X, Y: Integer;
begin
  if (CurBitmap = nil) then exit;
  if (CurBitmap= Glyph) and Glyph.Empty then exit;

  if FBtnAutosize and (CurBitmap<> Glyph) then
  begin
   Height:= CurBitmap.Height;
   Width:= CurBitmap.Width;
   ClyphRect:= ClientRect;
  end;

  CurBitmap.Transparent:= Transparent;
  CurBitmap.Modified:= False;

  if (csDesigning in ComponentState) then
  begin
   if not Enabled then
   begin
    if (CurBitmap<> Glyph) then
    begin
     if Stretch then Canvas.StretchDraw(ClyphRect, CurBitmap)
     else Canvas.Draw(GlyphPos.X, GlyphPos.Y, CurBitmap);
    end
    else
    begin
     DisabledBitmap:= CreateDisabledBitmap(CurBitmap, clBlack);
     Canvas.Draw(GlyphPos.X, GlyphPos.Y, DisabledBitmap);
    end;
   end
   else
   begin
    if Stretch then Canvas.StretchDraw(ClyphRect, CurBitmap)
    else Canvas.Draw(GlyphPos.X, GlyphPos.Y, CurBitmap);
   end;
   exit;
  end;

  if not Enabled then
  begin
   if (CurBitmap<> Glyph) then
   begin
    if Stretch then Canvas.StretchDraw(ClyphRect, CurBitmap)
    else Canvas.Draw(GlyphPos.X, GlyphPos.Y, CurBitmap);
   end
   else
   begin
    DisabledBitmap:= CreateDisabledBitmap(CurBitmap, clBlack);
    Canvas.Draw(GlyphPos.X, GlyphPos.Y, DisabledBitmap);
   end
  end
  else
  begin
   if (CurBitmap= Glyph) then
   begin
    if not FFocused and FGrayInactive then
    begin
     GrayBitmap.Assign(CurBitmap);
     with GrayBitmap do
     begin
      for X := 0 to Width - 1 do
          for Y := 0 to Height - 1 do
              Canvas.Pixels[X, Y] := MapColor(GrayBitmap.Canvas.Pixels[X, Y]);
     end;
     Canvas.Draw(GlyphPos.X, GlyphPos.Y, GrayBitmap);
    end
    else Canvas.Draw(GlyphPos.X, GlyphPos.Y, CurBitmap);
   end
   else Canvas.StretchDraw(ClyphRect, CurBitmap);
  end;
end;

procedure TSkinBitBtn.DrawButtonText(Canvas: TCanvas; const Caption: string;
  TextBounds: TRect; State: TButtonState; BiDiFlags: LongInt);
begin
  with Canvas do
  begin
    Brush.Style := bsClear;
    if State = bsDisabled then
    begin
      OffsetRect(TextBounds, 1, 1);
      Font.Color := clBtnHighlight;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
        DT_CENTER or DT_VCENTER or BiDiFlags);
      OffsetRect(TextBounds, -1, -1);
      Font.Color := clBtnShadow;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
        DT_CENTER or DT_VCENTER or BiDiFlags);
    end else
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
        DT_CENTER or DT_VCENTER or BiDiFlags);
  end;
end;

procedure TSkinBitBtn.CNDrawItem(var Message: TWMDrawItem);
begin
  DrawItem(Message.DrawItemStruct^);
end;

procedure TSkinBitBtn.ClearSkinBitBtn(Canvas: TCanvas; Rect: TRect);
begin
  Canvas.Pen.Color:= clBtnFace;
  Canvas.Brush.Color:= clBtnFace;
  Canvas.FillRect(Rect);
end;

procedure TSkinBitBtn.DrawItem(const DrawItemStruct: TDrawItemStruct);
var
  IsDown: Boolean;
  State: TButtonState;
  R: TRect;
  Offset: TPoint;
begin
  FCanvas.Handle:= DrawItemStruct.hDC;
  R:= ClientRect;

  with DrawItemStruct do
  begin
    IsDown:= itemState and ODS_SELECTED <> 0;
    if not Enabled then State:= bsDisabled
    else if IsDown then State:= bsDown
    else State:= bsUp;
  end;

  if State= bsDisabled then
  begin
   if not FSkinDisabled.Empty then
      DrawButtonGlyph(FCanvas, FSkinDisabled, R.TopLeft, State, True, True, R)
   else ClearSkinBitBtn(FCanvas, R);
  end
  else if FDown or IsDown then
       begin
        if not FSkinDown.Empty then
           DrawButtonGlyph(FCanvas, FSkinDown, R.TopLeft, State, True, True, R)
        else ClearSkinBitBtn(FCanvas, R);
       end
  else if FFocused then
       begin
        if not FSkinFocused.Empty then
           DrawButtonGlyph(FCanvas, FSkinFocused, R.TopLeft, State, True, True, R)
        else ClearSkinBitBtn(FCanvas, R);
       end
  else if Default then
       begin
        if not FFlat then
        begin
         if not FSkinDefault.Empty then
            DrawButtonGlyph(FCanvas, FSkinDefault, R.TopLeft, State, True, True, R)
         else ClearSkinBitBtn(FCanvas, R);
        end
        else
        begin
         if not FSkinFlat.Empty then
         DrawButtonGlyph(FCanvas, FSkinFlat, R.TopLeft, State, True, True, R)
         else ClearSkinBitBtn(FCanvas, R);
        end;
       end
  else
  begin
   if not FFlat then
   begin
    if not FSkinUp.Empty then
       DrawButtonGlyph(FCanvas, FSkinUp, R.TopLeft, State, True, True, R)
    else ClearSkinBitBtn(FCanvas, R);
   end
   else
   begin
    if not FSkinFlat.Empty then
       DrawButtonGlyph(FCanvas, FSkinFlat, R.TopLeft, State, True, True, R)
    else ClearSkinBitBtn(FCanvas, R);
   end;
  end;

  FCanvas.Font:= Font;
  if State in [bsDown, bsExclusive] then
  begin
   Offset.X:= 1;
   Offset.Y:= 0;
  end
  else
  begin
   Offset.X:= 0;
   Offset.Y:= -1;
  end;
  Draw(FCanvas, R, Offset, Caption, Layout, Margin,
    Spacing, State, DrawTextBiDiModeFlags(0));

  if not FFlat and not FFocused and not Default
  and Enabled and FMarkBtnFocused and Focused then
  begin
   R:= ClientRect;
   InflateRect(R, -5, -4);
   FCanvas.Pen.Color:= clWindowFrame;
   FCanvas.Brush.Color:= clBtnFace;
   DrawFocusRect(FCanvas.Handle, R);
  end;

  FCanvas.Handle:= 0;
end;

procedure TSkinBitBtn.SetGrayInactive(Value: Boolean);
begin
  if Value<> FGrayInactive then
  begin
   FGrayInactive:= Value;
   Invalidate;
  end;
  Repaint;
end;

procedure TSkinBitBtn.SetGlyphTransparent(Value: Boolean);
begin
  if Value<> FGlyphTransparent  Then
  begin
    FGlyphTransparent:= Value;
    Invalidate;
  end;
end;

procedure TSkinBitBtn.SetDown(Value: Boolean);
begin
  if Value<> FDown then
  begin
   FDown:= Value;
   Invalidate;
  end;
end;

procedure TSkinBitBtn.SetFlat(Value: Boolean);
begin
  if Value<> FFlat then
  begin
   FFlat:= Value;
   Invalidate;
  end;
end;

procedure TSkinBitBtn.SetBtnAutosize(Value: Boolean);
begin
  if Value<> FBtnAutosize then
  begin
   FBtnAutosize:= Value;
   Invalidate;
  end;
end;

function TSkinBitBtn.GetKind: TBitBtnKind;
begin
  if FKind <> bkCustom then
  begin
   if ((FKind in [bkOK, bkYes]) xor Default)
   or ((FKind in [bkCancel, bkNo]) xor Cancel)
   or (ModalResult <> SkinBitBtnModalResults[FKind])
   or Glyph.Modified then
      FKind:= bkCustom;
  end;
  Result:= FKind;
end;

procedure TSkinBitBtn.SetKind(Value: TBitBtnKind);
begin
  if Value<> FKind then
  begin
   if Value<> bkCustom then
   begin
    Default:= Value in [bkOK, bkYes];
    Cancel := Value in [bkCancel, bkNo];
    if ((csLoading in ComponentState) and (Caption= ''))
    or (not (csLoading in ComponentState)) then
    begin
     if SkinBitBtnCaptions[Value]<> nil then
        Caption:= LoadResString(SkinBitBtnCaptions[Value]);
    end;
    ModalResult:= SkinBitBtnModalResults[Value];
    NumGlyphs:= 1;
   end;
   if (csDesigning in ComponentState) then Glyph:= GetBitBtnGlyph(Value);
   Glyph.Modified:= False;
   FKind:= Value;
   Invalidate;
  end;
end;

procedure TSkinBitBtn.SetMarkBtnFocused(Value: Boolean);
begin
  if Value<> FMarkBtnFocused then
  begin
   FMarkBtnFocused:= Value;
   Invalidate;
  end;
end;

procedure TSkinBitBtn.SetSkinUp(Value: TBitmap);
begin
  FSkinUp.Assign(Value);
  Invalidate;
end;

procedure TSkinBitBtn.SetSkinDown(Value: TBitmap);
begin
  FSkinDown.Assign(Value);
  Invalidate;
end;

procedure TSkinBitBtn.SetSkinDisabled(Value: TBitmap);
begin
  FSkinDisabled.Assign(Value);
  Invalidate;
end;

procedure TSkinBitBtn.SetSkinDefault(Value: TBitmap);
begin
  FSkinDefault.Assign(Value);
  Invalidate;
end;

procedure TSkinBitBtn.SetSkinFocused(Value: TBitmap);
begin
  FSkinFocused.Assign(Value);
  Invalidate;
end;

procedure TSkinBitBtn.SetSkinFlat(Value: TBitmap);
begin
  FSkinFlat.Assign(Value);
  Invalidate;
end;

end.
