//************************************************************************
//* EloSoft components for Delphi 6 (copyright 2003)                     *
//************************************************************************

unit XPSpeedButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Buttons, Graphics,
  CommCtrl, Forms, Dialogs;

type
  TFillDirection = (fdTopToBottom, fdBottomToTop, fdLeftToRight, fdRightToLeft);
  TStyleButton = (xpsbsbRectangle, xpsbsbRoundRect, xpsbsbEllipse, xpsbsbClassic);
  TGradientKindEllipse = (gkBigAxis, gkSmallAxis);

  TXPSpeedButton = class(TSpeedButton)
  private
    { Private declarations }
    DisabledBitmap, GrayBitmap: TBitmap;
    FStyleButton: TStyleButton;
    FDefault: Boolean;
    FCancel: Boolean;
    FRadius: Word;
    FTransparent: Boolean;
    FFocused: Boolean;
    FGradient: Boolean;
    FGradientSteps: Word;
    FGradientMode: TFillDirection;
    FGradientEffectEllipse: Word;
    FGradientKindEllipse: TGradientKindEllipse;
    FExternalFrame: Boolean;
    FGrayInactive: Boolean;
    FDown: Boolean;
    FModalResult: TModalResult;

    FColorExternalFrameLeftTop: TColor;
    FColorExternalFrameRightBottom: TColor;

    FColorFrameLeftTop: TColor;
    FColorFrameRightBottom: TColor;

    FColorInsideFrameStart: TColor;
    FColorInsideFrameEnd: TColor;

    FColorDefaultButtonStart: TColor;
    FColorDefaultButtonEnd: TColor;

    FColorFocusedButtonStart: TColor;
    FColorFocusedButtonEnd: TColor;

    FGradientStart: TColor;
    FGradientEnd: TColor;

    procedure SetStyleButton(Value: TStyleButton);
    procedure SetDefault(Value: Boolean);
    procedure SetCancel(Value: Boolean);
    procedure SetGradient(Value: Boolean);
    procedure SetGradientSteps(Value: Word);
    procedure SetGradientMode(Value: TFillDirection);
    procedure SetGradientEffectEllipse(Value: Word);
    procedure SetGradientKindEllipse(Value: TGradientKindEllipse);
    procedure SetRadius(Value: Word);
    procedure SetTransparent(Value: Boolean);
    procedure SetExternalFrame(Value: Boolean);
    procedure SetGrayInactive(Value: Boolean);
    procedure SetDown(Value: Boolean);

    procedure SetColorExternalFrameLeftTop(AValue: TColor);
    procedure SetColorExternalFrameRightBottom(AValue: TColor);

    procedure SetFColorInsideFrameStart(AValue: TColor);
    procedure SetColorInsideFrameEnd(AValue: TColor);

    procedure SetColorDefaultButtonStart(AValue: TColor);
    procedure SetColorDefaultButtonEnd(AValue: TColor);

    procedure SetColorFocusedButtonStart(AValue: TColor);
    procedure SetColorFocusedButtonEnd(AValue: TColor);

    procedure SetColorFrameLeftTop(AValue: TColor);
    procedure SetColorFrameRightBottom(AValue: TColor);

    procedure SetGradientStart(AValue: TColor);
    procedure SetGradientEnd(AValue: TColor);
  protected
    { Protected declarations }
    function MapColor(Color: TColor): TColor;
    function Draw(Canvas: TCanvas; const Client: TRect;
      const Offset: TPoint; const Caption: string; Layout: TButtonLayout;
      Margin, Spacing: Integer; State: TButtonState; Transparent: Boolean;
      BiDiFlags: LongInt): TRect;
    procedure Paint; override;
    procedure CalcLayout(Canvas: TCanvas; const Client: TRect;
      const Offset: TPoint; const Caption: string; Layout: TButtonLayout; Margin,
      Spacing: Integer; var GlyphPos: TPoint; var TextBounds: TRect;
      BiDiFlags: LongInt);
    procedure DrawButtonGlyph(Canvas: TCanvas; const GlyphPos: TPoint;
      State: TButtonState; Transparent: Boolean);
    procedure DrawButtonText(Canvas: TCanvas; const Caption: string;
      TextBounds: TRect; State: TButtonState; BiDiFlags: LongInt);

    procedure PaintFrame(ACanvas: TCanvas;
      PenColorLeftTop, PenColorRightBottom: TColor;
      ALeft, ATop, AWidth, AHeight: Integer; ARadius: Word);
    procedure PaintInsideFrame(ACanvas: TCanvas;
      ALeft, ATop, AWidth, AHeight: Integer; ARadius: Word);

    procedure CMMouseLeave( var Message: TMessage ); Message CM_MOUSELEAVE;
    procedure CMMouseEnter( var Message: TMessage ); Message CM_MOUSEENTER;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    { Published declarations }
    property StyleButton: TStyleButton read FStyleButton write SetStyleButton;
    property Default: Boolean read FDefault write SetDefault;
    property Cancel: Boolean read FCancel write SetCancel default False;
    property GradientSteps: Word read FGradientSteps write SetGradientSteps;
    property GradientMode: TFillDirection read FGradientMode write SetGradientMode;
    property GradientEffectEllipse: Word read FGradientEffectEllipse write SetGradientEffectEllipse;
    property GradientKindEllipse: TGradientKindEllipse read FGradientKindEllipse write SetGradientKindEllipse;
    property Gradient: Boolean read FGradient write SetGradient;
    property ExternalFrame: Boolean read FExternalFrame write SetExternalFrame;
    property GrayInactive: Boolean read FGrayInactive write SetGrayInactive;
    property Radius: Word read FRadius write SetRadius;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Down: Boolean read FDown write SetDown;
    property ModalResult: TModalResult read FModalResult write FModalResult default 0;

    property ColorExternalFrameLeftTop: TColor read FColorExternalFrameLeftTop write SetColorExternalFrameLeftTop;
    property ColorExternalFrameRightBottom: TColor read FColorExternalFrameRightBottom write SetColorExternalFrameRightBottom;

    property ColorInsideFrameStart: TColor read FColorInsideFrameStart write SetFColorInsideFrameStart;
    property ColorInsideFrameEnd: TColor read FColorInsideFrameEnd write SetColorInsideFrameEnd;

    property ColorDefaultButtonStart: TColor read FColorDefaultButtonStart write SetColorDefaultButtonStart;
    property ColorDefaultButtonEnd: TColor read FColorDefaultButtonEnd write SetColorDefaultButtonEnd;

    property ColorFocusedButtonStart: TColor read FColorFocusedButtonStart write SetColorFocusedButtonStart;
    property ColorFocusedButtonEnd: TColor read FColorFocusedButtonEnd write SetColorFocusedButtonEnd;

    property ColorFrameLeftTop: TColor read FColorFrameLeftTop write SetColorFrameLeftTop;
    property ColorFrameRightBottom: TColor read FColorFrameRightBottom write SetColorFrameRightBottom;

    property GradientStart: TColor read FGradientStart write SetGradientStart;
    property GradientEnd: TColor read FGradientEnd write SetGradientEnd;
  end;

procedure Register;

implementation

function Max(A, B: Longint): Longint;
begin
  if A > B then Result := A
  else Result := B;
end;

function Min(A, B: Longint): Longint;
begin
  if A < B then Result := A
  else Result := B;
end;

function HeightOf(R: TRect): Integer;
begin
  Result := R.Bottom - R.Top;
end;

function WidthOf(R: TRect): Integer;
begin
  Result := R.Right - R.Left;
end;

function CreateDisabledBitmapEx(FOriginal: TBitmap; OutlineColor, BackColor,
  HighlightColor, ShadowColor: TColor; DrawHighlight: Boolean): TBitmap;
var
  MonoBmp: TBitmap;
  IRect: TRect;
begin
  IRect := Rect(0, 0, FOriginal.Width, FOriginal.Height);
  Result := TBitmap.Create;
  try
    Result.Width := FOriginal.Width;
    Result.Height := FOriginal.Height;
    MonoBmp := TBitmap.Create;
    try
      with MonoBmp do begin
        Width := FOriginal.Width;
        Height := FOriginal.Height;
        Canvas.CopyRect(IRect, FOriginal.Canvas, IRect);
{$IFDEF RX_D3}
        HandleType := bmDDB;
{$ENDIF}
        Canvas.Brush.Color := OutlineColor;
        if Monochrome then begin
          Canvas.Font.Color := clWhite;
          Monochrome := False;
          Canvas.Brush.Color := clWhite;
        end;
        Monochrome := True;
      end;
      with Result.Canvas do begin
        Brush.Color := BackColor;
        FillRect(IRect);
        if DrawHighlight then begin
          Brush.Color := HighlightColor;
          SetTextColor(Handle, clBlack);
          SetBkColor(Handle, clWhite);
          BitBlt(Handle, 1, 1, WidthOf(IRect), HeightOf(IRect),
            MonoBmp.Canvas.Handle, 0, 0, $00E20746);
        end;
        Brush.Color := ShadowColor;
        SetTextColor(Handle, clBlack);
        SetBkColor(Handle, clWhite);
        BitBlt(Handle, 0, 0, WidthOf(IRect), HeightOf(IRect),
          MonoBmp.Canvas.Handle, 0, 0, $00E20746);
      end;
    finally
      MonoBmp.Free;
    end;
  except
    Result.Free;
    raise;
  end;
end;

function CreateDisabledBitmap(FOriginal: TBitmap; OutlineColor: TColor): TBitmap;
begin
  Result := CreateDisabledBitmapEx(FOriginal, OutlineColor,
    clBtnFace, clBtnHighlight, clBtnShadow, True);
end;

function GetCColor(Color01,Color02: TColor; R, I: Integer): TColor;
var
  C1,C2 : Tcolor;
  R1,G1,B1,R2,G2,B2 : Byte;
begin
  c1:=ColorTorgb(Color01);
  c2:=ColorTorgb(Color02);
  R1:=Pbyte(@C1)^;
  G1:=Pbyte(integer(@C1)+1)^;
  B1:=Pbyte(integer(@C1)+2)^;
  R2:=Pbyte(@C2)^;
  G2:=Pbyte(integer(@C2)+1)^;
  B2:=Pbyte(integer(@C2)+2)^;
  if R<>0 then
     Result:=RGB((R1+(R2-R1)*i div R),(G1+(G2-G1)*i div R),(B1+(B2-B1)*i div R))
  else Result:=Color01;
end;

{ Gradient fill procedure - displays a gradient beginning with a chosen    }
{ color and ending with another chosen color. Based on TGradientFill       }
{ component source code written by Curtis White, cwhite@teleport.com.      }
{   EloSoft (2003) asv4416@oues.ru                     }
procedure GradientFillRoundRect(ACanvas: TCanvas; ARect: TRect; StartColor,
  EndColor: TColor; Direction: TFillDirection; Colors: Byte; ARadius: Word;
  IncreaseFirstLastLine: Boolean; ValueIncrease: Word; Invert: Boolean;
  AStyleButton: TStyleButton; AGradientEffectEllipse: Word;
  AGradientKindEllipse: TGradientKindEllipse);
var
  StartRGB: array[0..2] of Byte;    { Start RGB values }
  RGBDelta: array[0..2] of Integer; { Difference between start and end RGB values }
  ColorBand: TRect;                 { Color band rectangular coordinates }
  I, Delta, K, L, W: Integer;
  Brush: HBrush;
begin
 case AStyleButton of
 xpsbsbRectangle, xpsbsbRoundRect:
 begin
  if StartColor=EndColor then
  begin
   ACanvas.Pen.Color:= StartColor;
   ACanvas.Brush.Color:= ACanvas.Pen.Color;
   ACanvas.RoundRect(ARect.Left, ARect.Top, ARect.Right, ARect.Bottom, ARadius, ARadius);
   exit;
  end;

  ARadius:= ARadius div 2;

  if Invert then
  begin
   if Direction= fdTopToBottom then Direction:= fdBottomToTop
   else if Direction= fdBottomToTop then Direction:= fdTopToBottom
   else if Direction= fdLeftToRight then Direction:= fdRightToLeft
   else if Direction= fdRightToLeft then Direction:= fdLeftToRight;
  end;

  if IsRectEmpty(ARect) then Exit;

  StartColor := ColorToRGB(StartColor);
  EndColor := ColorToRGB(EndColor);
  case Direction of
    fdTopToBottom, fdLeftToRight: begin
      { Set the Red, Green and Blue colors }
      StartRGB[0] := GetRValue(StartColor);
      StartRGB[1] := GetGValue(StartColor);
      StartRGB[2] := GetBValue(StartColor);
      { Calculate the difference between begin and end RGB values }
      RGBDelta[0] := GetRValue(EndColor) - StartRGB[0];
      RGBDelta[1] := GetGValue(EndColor) - StartRGB[1];
      RGBDelta[2] := GetBValue(EndColor) - StartRGB[2];
    end;
    fdBottomToTop, fdRightToLeft: begin
      { Set the Red, Green and Blue colors }
      { Reverse of TopToBottom and LeftToRight directions }
      StartRGB[0] := GetRValue(EndColor);
      StartRGB[1] := GetGValue(EndColor);
      StartRGB[2] := GetBValue(EndColor);
      { Calculate the difference between begin and end RGB values }
      { Reverse of TopToBottom and LeftToRight directions }
      RGBDelta[0] := GetRValue(StartColor) - StartRGB[0];
      RGBDelta[1] := GetGValue(StartColor) - StartRGB[1];
      RGBDelta[2] := GetBValue(StartColor) - StartRGB[2];
    end;
  end; {case}
  { Calculate the color band's coordinates }
  ColorBand := ARect;
  if Direction in [fdTopToBottom, fdBottomToTop] then begin
    Colors := Max(2, Min(Colors, HeightOf(ARect)));
    Delta := HeightOf(ARect) div Colors;
  end
  else begin
    Colors := Max(2, Min(Colors, WidthOf(ARect)));
    Delta := WidthOf(ARect) div Colors;
  end;
  with ACanvas.Pen do begin { Set the pen style and mode }
    Style := psSolid;
    Mode := pmCopy;
  end;
  { Perform the fill }
  if Delta > 0 then begin
    for I := 0 to Colors- 1 do begin
      case Direction of
        { Calculate the color band's top and bottom coordinates }
        fdTopToBottom, fdBottomToTop:
        begin
          ColorBand.Top := ARect.Top + I * Delta;
          ColorBand.Bottom := ColorBand.Top + Delta;
          ColorBand.Left:= ARect.Left;
          ColorBand.Right:= ARect.Right;

          if ARadius> 0 then
          begin
           if ColorBand.Top- ARect.Top< ARadius then
           begin
            ColorBand.Left:= trunc(ColorBand.Left+ ARadius- sqrt(abs(sqr(ARadius)- sqr(ARadius- ColorBand.Top+ ARect.Top))));
            if (i= 0) and IncreaseFirstLastLine then
               ColorBand.Left:= ColorBand.Left- ValueIncrease;
            ColorBand.Right:= ColorBand.Right-(ColorBand.Left- ARect.Left);
           end;
           if ARect.Bottom- ColorBand.Bottom< ARadius then
           begin
            ColorBand.Left:= trunc(ColorBand.Left+ ARadius- sqrt(abs(sqr(ARadius)- sqr(ARadius- ARect.Bottom+ ColorBand.Bottom))));
            if (i= Colors- 1) and IncreaseFirstLastLine then
               ColorBand.Left:= ColorBand.Left- ValueIncrease;
            ColorBand.Right:= ColorBand.Right-(ColorBand.Left- ARect.Left);
           end;
          end;

        end;
        { Calculate the color band's left and right coordinates }
        fdLeftToRight, fdRightToLeft:
        begin
          ColorBand.Left := ARect.Left + I * Delta;
          ColorBand.Right := ColorBand.Left + Delta;
          ColorBand.Top:= ARect.Top;
          ColorBand.Bottom:= ARect.Bottom;

          if ARadius> 0 then
          begin
           if ColorBand.Left- ARect.Left< ARadius then
           begin
            ColorBand.Top:= trunc(ColorBand.Top+ ARadius- sqrt(abs(sqr(ARadius)- sqr(ARadius- ColorBand.Left+ ARect.Left))));
            if (i= 0) and IncreaseFirstLastLine then
               ColorBand.Top:= ColorBand.Top- ValueIncrease;
            ColorBand.Bottom:= ColorBand.Bottom-(ColorBand.Top- ARect.Top);
           end;
           if ARect.Right- ColorBand.Right< ARadius then
           begin
            ColorBand.Top:= trunc(ColorBand.Top+ ARadius- sqrt(abs(sqr(ARadius)- sqr(ARadius- ARect.Right+ ColorBand.Right))));
            if (i= Colors- 1) and IncreaseFirstLastLine then
               ColorBand.Top:= ColorBand.Top- ValueIncrease;
            ColorBand.Bottom:= ColorBand.Bottom-(ColorBand.Top- ARect.Top);
           end;
          end;

        end;
      end; {case}
      { Calculate the color band's color }
      Brush := CreateSolidBrush(RGB(
        StartRGB[0] + MulDiv(I, RGBDelta[0], Colors - 1),
        StartRGB[1] + MulDiv(I, RGBDelta[1], Colors - 1),
        StartRGB[2] + MulDiv(I, RGBDelta[2], Colors - 1)));
      FillRect(ACanvas.Handle, ColorBand, Brush);
      DeleteObject(Brush);
    end;
  end;
  if Direction in [fdTopToBottom, fdBottomToTop] then
    Delta := HeightOf(ARect) mod Colors
  else Delta := WidthOf(ARect) mod Colors;
  if Delta > 0 then begin
    case Direction of
      { Calculate the color band's top and bottom coordinates }
      fdTopToBottom, fdBottomToTop: begin
        ColorBand.Top := ARect.Bottom - Delta;
        ColorBand.Bottom := ColorBand.Top + Delta;
      end;
      { Calculate the color band's left and right coordinates }
      fdLeftToRight, fdRightToLeft: begin
        ColorBand.Left := ARect.Right - Delta;
        ColorBand.Right := ColorBand.Left + Delta;
      end;
    end; {case}
    case Direction of
      fdTopToBottom, fdLeftToRight:
        Brush := CreateSolidBrush(EndColor);
      else {fdBottomToTop, fdRightToLeft }
        Brush := CreateSolidBrush(StartColor);
    end;
    FillRect(ACanvas.Handle, ColorBand, Brush);
    DeleteObject(Brush);
  end;
 end;

 xpsbsbEllipse:
 begin
  if StartColor=EndColor then
  begin
   ACanvas.Pen.Color:= StartColor;
   ACanvas.Brush.Color:= ACanvas.Pen.Color;
   ACanvas.Ellipse(ARect.Left, ARect.Top, ARect.Right, ARect.Bottom);
   exit;
  end;

  if Invert then
  begin
   StartColor := ColorToRGB(StartColor);
   StartColor:= GetCColor(StartColor, EndColor, 17, AGradientEffectEllipse);
   EndColor := ColorToRGB(EndColor);
   EndColor:= GetCColor(StartColor, EndColor, 7, -AGradientEffectEllipse);
  end
  else
  begin
   StartColor := ColorToRGB(StartColor);
   EndColor := ColorToRGB(EndColor);
  end;

  K:= 0;
  W:= ARect.Right- ARect.Left;
  L:= ARect.Bottom- ARect.Top;
  case AGradientKindEllipse of
   gkBigAxis:   if L> W then K:= L else K:= W;
   gkSmallAxis: if L> W then K:= W else K:= L;
  end;

  for I:= 0 to (K div 2)- 1 do
  begin
   ACanvas.Pen.Color:=GetCColor(StartColor, EndColor, I, AGradientEffectEllipse);
   ACanvas.Brush.Color:= ACanvas.Pen.Color;
   ACanvas.Ellipse(ARect.Left, ARect.Top, ARect.Right, ARect.Bottom);
   ARect.Left:= Trunc(ARect.Left+ 1);
   ARect.Top:= Trunc(ARect.Top+ 1);
   ARect.Right:= Trunc(ARect.Right- 1);
   ARect.Bottom:= Trunc(ARect.Bottom- 1);
  end;

 end;
 end;
end;

constructor TXPSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  GrayBitmap:= TBitmap.Create;
  DisabledBitmap:= TBitmap.Create;
  FFocused:=False;
  FTransparent:= True;
  FRadius:= 2;
  FStyleButton:= xpsbsbRoundRect;
  FDefault:= False;
  FCancel:= False;
  FDown:= False;
  FGradientSteps:= 255;
  FGradientMode:= fdTopToBottom;
  FGradientEffectEllipse:= 3;
  FGradientKindEllipse:= gkSmallAxis;
  FGradient:= True;
  FExternalFrame:= True;
  FGrayInactive:= True;
  FModalResult:= 0;

  FColorExternalFrameLeftTop:= clBtnShadow;
  FColorExternalFrameRightBottom:= cl3DLight;

  FColorInsideFrameStart:= clWhite;
  FColorInsideFrameEnd:= clWhite;

  FColorDefaultButtonStart:= $00FFE9B9;
  FColorDefaultButtonEnd:= $00F4A500;

  FColorFocusedButtonStart:= $00C0F1FE;
  FColorFocusedButtonEnd:= $0037B9FF;

  FColorFrameLeftTop:= clNavy;
  FColorFrameRightBottom:= clNavy;

  FGradientStart:= clWhite;
  FGradientEnd:= $00C9B8B4;
end;

destructor TXPSpeedButton.Destroy;
begin
  if GrayBitmap<> nil then GrayBitmap.Free;
  GrayBitmap:= nil;
  if DisabledBitmap<> nil then DisabledBitmap.Free;
  DisabledBitmap:= nil;
  
  inherited Destroy;
end;

procedure TXPSpeedButton.SetStyleButton(Value: TStyleButton);
begin
  if Value<> FStyleButton then
  begin
   FStyleButton:= Value;
   if (FStyleButton= xpsbsbRectangle) or (FStyleButton= xpsbsbEllipse) then FRadius:= 0
   else if FRadius= 0 then FRadius:= 2;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetDefault(Value: Boolean);
var
  Form: TCustomForm;
begin
  FDefault := Value;
  if Parent<> nil then
  begin
    Form := GetParentForm(Self);
    if Form <> nil then
      Form.Perform(CM_FOCUSCHANGED, 0, Longint(Form.ActiveControl));
  end;
  if FDefault then FCancel:= False;
  Invalidate;
end;

procedure TXPSpeedButton.SetCancel(Value: Boolean);
begin
  if Value<> FCancel then
  begin
   FCancel:= Value;
   if FCancel then
   begin
    SetDefault(False);
    ModalResult:= mrCancel;
   end;
  end;
end;

procedure TXPSpeedButton.SetGradient(Value: Boolean);
begin
  if Value<> FGradient then
  begin
   FGradient:= Value;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetGradientSteps(Value: Word);
begin
  if Value> 255 then
     MessageDlg(' GradientSteps '+ #13+
                '    0  255',
                mtError, [mbOk], 0)
  else
  begin
   if Value<> FGradientSteps then
   begin
    FGradientSteps:= Value;
    Invalidate;
   end;
  end;
end;

procedure TXPSpeedButton.SetGradientMode(Value: TFillDirection);
begin
  if Value <> FGradientMode then
  begin
   FGradientMode:= Value;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetGradientEffectEllipse(Value: Word);
begin
  if Value <> FGradientEffectEllipse then
  begin
   FGradientEffectEllipse:= Value;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetGradientKindEllipse(Value: TGradientKindEllipse);
begin
  if Value <> FGradientKindEllipse then
  begin
   FGradientKindEllipse:= Value;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetColorExternalFrameLeftTop(AValue: TColor);
begin
  if FColorExternalFrameLeftTop<> AValue then
  begin
   FColorExternalFrameLeftTop:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetColorExternalFrameRightBottom(AValue: TColor);
begin
  if FColorExternalFrameRightBottom<> AValue then
  begin
   FColorExternalFrameRightBottom:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetFColorInsideFrameStart(AValue: TColor);
begin
  if FColorInsideFrameStart<> AValue then
  begin
   FColorInsideFrameStart:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetColorInsideFrameEnd(AValue: TColor);
begin
  if FColorInsideFrameEnd<> AValue then
  begin
   FColorInsideFrameEnd:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetColorDefaultButtonStart(AValue: TColor );
begin
  if FColorDefaultButtonStart<> AValue then
  begin
   FColorDefaultButtonStart:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetColorDefaultButtonEnd(AValue: TColor );
begin
  if FColorDefaultButtonEnd<> AValue then
  begin
   FColorDefaultButtonEnd:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetColorFocusedButtonStart(AValue: TColor );
begin
  if FColorFocusedButtonStart<> AValue then
  begin
   FColorFocusedButtonStart:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetColorFocusedButtonEnd(AValue: TColor );
begin
  if FColorFocusedButtonEnd<> AValue then
  begin
   FColorFocusedButtonEnd:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetColorFrameLeftTop(AValue: TColor );
begin
  if FColorFrameLeftTop<> AValue then
  begin
   FColorFrameLeftTop:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetColorFrameRightBottom(AValue: TColor );
begin
  if FColorFrameRightBottom<> AValue then
  begin
   FColorFrameRightBottom:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetGradientStart(AValue: TColor );
begin
  if FGradientStart<> AValue then
  begin
   FGradientStart:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetGradientEnd(AValue: TColor );
begin
  if FGradientEnd<> AValue then
  begin
   FGradientEnd:= AValue;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetRadius(Value: Word);
begin
  if (FStyleButton= xpsbsbRectangle) or (FStyleButton= xpsbsbEllipse) then
     Value:= 0;

  if Value> (Height div 2)- 2 then
  begin
   MessageDlg(' Radius  '+ #13+
              '  (Height div 2)-2',
              mtError, [mbOk], 0);
   Value:= (Height div 2)- 2;
  end;

  if Value<> FRadius then
  begin
   FRadius:= Value;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetTransparent(Value: Boolean);
begin
  if FTransparent<> Value Then
  begin
    FTransparent:= Value;
    Invalidate;
  end;
end;

procedure TXPSpeedButton.SetExternalFrame(Value: Boolean);
begin
  if Value<> FExternalFrame then
  begin
   FExternalFrame:= Value;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetGrayInactive(Value: Boolean);
begin
  if Value<> FGrayInactive then
  begin
   FGrayInactive:= Value;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.SetDown(Value: Boolean);
begin
  if Value<> FDown then
  begin
   FDown:= Value;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.CMMouseLeave( Var Message: TMessage );
begin
  inherited;
  if FFocused then
  begin
   FFocused:=False;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.CMMouseEnter( Var Message: TMessage );
begin
  inherited;
  if not FFocused then
  begin
   FFocused:=True;
   Invalidate;
  end;
end;

procedure TXPSpeedButton.Click;
var
  Form: TCustomForm;
begin
  Form := GetParentForm(Self);
  if Form <> nil then Form.ModalResult := ModalResult;
  inherited Click;
end;

procedure TXPSpeedButton.PaintFrame(ACanvas: TCanvas;
  PenColorLeftTop, PenColorRightBottom: TColor;
  ALeft, ATop, AWidth, AHeight: Integer; ARadius: Word);
begin
  if not (csDesigning in ComponentState) then
     if Flat and not FFocused then exit;
  ACanvas.Pen.Style:= psSolid;
  ACanvas.Brush.Style:= bsSolid;
  // 
  ACanvas.Pen.Color:= PenColorRightBottom;
  ACanvas.Brush.Color:= ACanvas.Pen.Color;
  if FStyleButton= xpsbsbEllipse then
     ACanvas.Ellipse(ALeft, ATop, AWidth, AHeight)
  else ACanvas.RoundRect(ALeft, ATop, AWidth, AHeight, ARadius, ARadius);
 // 
  ACanvas.Pen.Color:= PenColorLeftTop;
  ACanvas.Brush.Color:= ACanvas.Pen.Color;
  if FStyleButton= xpsbsbEllipse then
     ACanvas.Ellipse(ALeft, ATop, AWidth- 1, AHeight- 1)
  else ACanvas.RoundRect(ALeft, ATop, AWidth- 1, AHeight- 1, ARadius, ARadius);
end;

procedure TXPSpeedButton.PaintInsideFrame(ACanvas: TCanvas;
  ALeft, ATop, AWidth, AHeight: Integer; ARadius: Word);
var
   GradientRect: TRect;
   StartColor, EndColor: TColor;
   Invert: Boolean;
begin
  if not (csDesigning in ComponentState) then
     if Flat and not FFocused then exit;
  if FState= bsDisabled then exit;

  if (FState= bsDown) or FDown then Invert:= True
  else Invert:= False;

  ACanvas.Pen.Style:= psSolid;
  ACanvas.Brush.Style:= bsSolid;

  // 
  if FFocused then
  begin
   if not (FState= bsDown) then
   begin
    StartColor:= FColorFocusedButtonStart;
    EndColor:= FColorFocusedButtonEnd;
   end
   else
   begin
    StartColor:= FColorInsideFrameStart;
    EndColor:= FColorInsideFrameEnd;
   end;
  end
  else
  begin
   if FDefault then
   begin
    StartColor:= FColorDefaultButtonStart;
    EndColor:= FColorDefaultButtonEnd;
   end
   else
   begin
    StartColor:= FColorInsideFrameStart;
    EndColor:= FColorInsideFrameEnd;
   end;
  end;

  if FGradient then
  begin
   if FStyleButton= xpsbsbEllipse then
   begin
    if FFocused or FDefault then
    begin
     ACanvas.Pen.Color:= StartColor;
     ACanvas.Brush.Color:= clBtnFace;
     ACanvas.Ellipse(ALeft, ATop- 1, AWidth, AHeight+ 1);
    end;
   end
   else
   begin
    GradientRect.Left:= ALeft;
    GradientRect.Top:= ATop;
    GradientRect.Right:= AWidth;
    GradientRect.Bottom:= AHeight;
    GradientFillRoundRect(ACanvas, GradientRect, StartColor, EndColor,
      FGradientMode, FGradientSteps, ARadius, True, 2, Invert,
      FStyleButton, FGradientEffectEllipse, FGradientKindEllipse)
   end;
  end // end if FGradient
  else
  begin
   ACanvas.Pen.Color:= StartColor;
   ACanvas.Brush.Color:= clBtnFace;
   if FStyleButton= xpsbsbEllipse then
   begin
    if FFocused or FDefault then ACanvas.Ellipse(ALeft, ATop, AWidth, AHeight);
   end
   else
   begin
    ACanvas.RoundRect(ALeft, ATop, AWidth, AHeight, ARadius, ARadius);
    ACanvas.Pen.Color:= EndColor;
    ACanvas.Brush.Color:= EndColor;
    ACanvas.RoundRect(ALeft, ATop, AWidth- 1, AHeight- 1, ARadius, ARadius);
   end;
  end;

  // 
  if FFocused or FDefault then
  begin
   if FGradient then
   begin
    // 
    if FStyleButton<> xpsbsbEllipse then
    begin
     if FFocused then
     begin
      GradientRect.Left:= ALeft+ 2;
      GradientRect.Right:= AWidth- 2;
     end
     else
     begin
      GradientRect.Left:= ALeft+ 1;
      GradientRect.Right:= AWidth- 1;
     end;
     GradientRect.Top:= ATop+ 2;
     GradientRect.Bottom:= AHeight- 2;
     GradientFillRoundRect(ACanvas, GradientRect, FColorInsideFrameStart,
       FColorInsideFrameEnd, FGradientMode, FGradientSteps, ARadius,
       True, 2, Invert, FStyleButton, FGradientEffectEllipse, FGradientKindEllipse);
    end;
   end // if FGradient
   else
   begin
    ACanvas.Pen.Color:= FColorInsideFrameStart;
    ACanvas.Brush.Color:= clBtnFace;
    if FStyleButton= xpsbsbEllipse then
    begin
     if FDefault then ACanvas.Ellipse(ALeft+ 2, ATop+ 2, AWidth- 2, AHeight- 2)
     else ACanvas.Ellipse(ALeft+ 1, ATop+ 1, AWidth- 1, AHeight- 1);
    end
    else
    begin
     if FDefault and not FFocused then
        ACanvas.RoundRect(ALeft+ 1, ATop+ 2, AWidth- 1, AHeight- 2, ARadius, ARadius)
     else ACanvas.RoundRect(ALeft+ 2, ATop+ 2, AWidth- 2, AHeight- 2, ARadius, ARadius);
    end;
   end;

   //  
   if FGradient then
   begin
    if FStyleButton= xpsbsbEllipse then
    begin
     GradientRect.Left:= ALeft+ 1;
     GradientRect.Top:= ATop;
     GradientRect.Right:= AWidth- 1;
     GradientRect.Bottom:= AHeight;
    end
    else
    begin
     if FFocused then
     begin
      if not (FState= bsDown) then
      begin
       GradientRect.Left:= ALeft+ 3;
       GradientRect.Top:= ATop+ 2;
       GradientRect.Right:= AWidth- 3;
       GradientRect.Bottom:= AHeight- 2;
      end
      else
      begin
       GradientRect.Left:= ALeft+ 1;
       GradientRect.Top:= ATop+ 1;
       GradientRect.Right:= AWidth- 1;
       GradientRect.Bottom:= AHeight;
      end;
     end
     else
      begin
      GradientRect.Left:= ALeft+ 2;
      GradientRect.Top:= ATop+ 2;
      GradientRect.Right:= AWidth- 2;
      GradientRect.Bottom:= AHeight- 2;
     end;
    end;
    GradientFillRoundRect(ACanvas, GradientRect, FGradientStart,
      FGradientEnd, FGradientMode, FGradientSteps, ARadius, True, 2,
      Invert, FStyleButton, FGradientEffectEllipse, FGradientKindEllipse);
   end // end if FGradient
   else
   begin
    ACanvas.Pen.Color:= FGradientStart;
    if Invert then ACanvas.Brush.Color:= clBtnShadow
    else ACanvas.Brush.Color:= clBtnFace;
    if FStyleButton= xpsbsbEllipse then
       ACanvas.Ellipse(ALeft+ 1, ATop+ 1, AWidth- 1, AHeight- 1)
    else
    begin
     if (FState= bsDown) or FDown then
        ACanvas.RoundRect(ALeft+ 1, ATop, AWidth- 1, AHeight, ARadius, ARadius)
     else
     if FDefault and not FFocused then
        ACanvas.RoundRect(ALeft+ 1, ATop+ 2, AWidth- 1, AHeight- 2, ARadius, ARadius)
     else ACanvas.RoundRect(ALeft+ 2, ATop+ 2, AWidth- 2, AHeight- 2, ARadius, ARadius);
    end;
   end;
  end // if FFocused or FDefault
  else
  begin
   if FGradient then
   begin
    GradientRect.Left:= 3;
    GradientRect.Top:= 2;
    GradientRect.Right:= Width- 3;
    GradientRect.Bottom:= Height- 2;
    GradientFillRoundRect(ACanvas, GradientRect, FGradientStart,
      FGradientEnd, FGradientMode, FGradientSteps, ARadius, True, 2,
      Invert, FStyleButton, FGradientEffectEllipse, FGradientKindEllipse);
   end
   else
   begin
    ACanvas.Pen.Color:= FGradientStart;
    if Invert then ACanvas.Brush.Color:= clBtnShadow
    else ACanvas.Brush.Color:= clBtnFace;
    if FStyleButton= xpsbsbEllipse then
       ACanvas.Ellipse(ALeft, ATop, AWidth, AHeight)
    else ACanvas.RoundRect(ALeft+ 2, ATop+ 1, AWidth- 2, AHeight- 1, ARadius, ARadius);
   end;
  end;
end;

function TXPSpeedButton.Draw(Canvas: TCanvas; const Client: TRect;
  const Offset: TPoint; const Caption: string; Layout: TButtonLayout;
  Margin, Spacing: Integer; State: TButtonState; Transparent: Boolean;
  BiDiFlags: LongInt): TRect;
var
  GlyphPos: TPoint;
begin
  CalcLayout(Canvas, Client, Offset, Caption, Layout, Margin, Spacing,
    GlyphPos, Result, BiDiFlags);
  DrawButtonGlyph(Canvas, GlyphPos, FState, Transparent);
  DrawButtonText(Canvas, Caption, Result, State, BiDiFlags);
end;

procedure TXPSpeedButton.Paint;
var
  PaintRect: TRect;
  Offset: TPoint;
  CurRadius: Word;
begin
   if FStyleButton= xpsbsbClassic then
   begin
    inherited Paint;
    exit;
   end;

   if not (csDesigning in ComponentState) and not Visible then exit;

   if (FStyleButton= xpsbsbRectangle) or (FStyleButton= xpsbsbEllipse) then FRadius:= 0;
   if FRadius> (Height div 2)- 2 then FRadius:= (Height div 2)- 2;
   CurRadius:= 2* FRadius;

   if not Enabled then FState := bsDisabled
   else if FState = bsDisabled then
           if Down and (GroupIndex <> 0) then
              FState := bsExclusive
           else FState := bsUp;

   Canvas.Font := Self.Font;
   PaintRect := Rect(0, 0, Width, Height);

   if FState= bsDisabled then
   begin
    // 
    if FExternalFrame then
       PaintFrame(Canvas, FColorExternalFrameLeftTop, FColorExternalFrameRightBottom, 0, 0, width, Height, CurRadius);
    //
    PaintFrame(Canvas, clInactiveBorder, clBtnFace, 1, 1, Width- 1, Height- 1, CurRadius);
    //   
    PaintFrame(Canvas, clBtnFace, clInactiveBorder, 2, 2, Width- 2, Height- 2, CurRadius);
   end
   else
   begin
    // 
    if FExternalFrame then
       PaintFrame(Canvas, FColorExternalFrameLeftTop, FColorExternalFrameRightBottom, 0, 0, width, Height, CurRadius);
    //
    PaintFrame(Canvas, FColorFrameLeftTop, FColorFrameRightBottom, 1, 1, Width- 1, Height- 1, CurRadius);
    //   
    PaintInsideFrame(Canvas, 2, 2, Width- 2, Height- 2, CurRadius);
   end;

   if FState in [bsDown, bsExclusive] then
   begin
    Offset.X := 1;
    Offset.Y := 0;
   end
   else
   begin
    Offset.X := 0;
    Offset.Y := -1;
   end;

   Draw(Canvas, PaintRect, Offset, Caption, Layout, Margin,
     Spacing, FState, Transparent, DrawTextBiDiModeFlags(0));
end;

procedure TXPSpeedButton.CalcLayout(Canvas: TCanvas; const Client: TRect;
  const Offset: TPoint; const Caption: string; Layout: TButtonLayout; Margin,
  Spacing: Integer; var GlyphPos: TPoint; var TextBounds: TRect;
  BiDiFlags: LongInt);
var
  TextPos: TPoint;
  ClientSize, GlyphSize, TextSize: TPoint;
  TotalSize: TPoint;
begin
  if (BiDiFlags and DT_RIGHT) = DT_RIGHT then
    if Layout = blGlyphLeft then Layout := blGlyphRight
    else
      if Layout = blGlyphRight then Layout := blGlyphLeft;
  { calculate the item sizes }
  ClientSize := Point(Client.Right - Client.Left, Client.Bottom -
    Client.Top);

  if Glyph <> nil then
    GlyphSize := Point(Glyph.Width div NumGlyphs, Glyph.Height) else
    GlyphSize := Point(0, 0);
    
  if Length(Caption) > 0 then
  begin
    TextBounds := Rect(0, 0, Client.Right - Client.Left, 0);
    DrawText(Canvas.Handle, PChar(Caption), Length(Caption), TextBounds,
      DT_CALCRECT or BiDiFlags);
    TextSize := Point(TextBounds.Right - TextBounds.Left, TextBounds.Bottom -
      TextBounds.Top);
  end
  else
  begin
    TextBounds := Rect(0, 0, 0, 0);
    TextSize := Point(0,0);
  end;
    
  { If the layout has the glyph on the right or the left, then both the
    text and the glyph are centered vertically.  If the glyph is on the top
    or the bottom, then both the text and the glyph are centered horizontally.}
  if Layout in [blGlyphLeft, blGlyphRight] then
  begin
    GlyphPos.Y := (ClientSize.Y - GlyphSize.Y + 1) div 2;
    TextPos.Y := (ClientSize.Y - TextSize.Y + 1) div 2;
  end
  else
  begin
    GlyphPos.X := (ClientSize.X - GlyphSize.X + 1) div 2;
    TextPos.X := (ClientSize.X - TextSize.X + 1) div 2;
  end;

  { if there is no text or no bitmap, then Spacing is irrelevant }
  if (TextSize.X = 0) or (GlyphSize.X = 0) then
    Spacing := 0;

  { adjust Margin and Spacing }
  if Margin = -1 then
  begin
    if Spacing = -1 then
    begin
      TotalSize := Point(GlyphSize.X + TextSize.X, GlyphSize.Y + TextSize.Y);
      if Layout in [blGlyphLeft, blGlyphRight] then
        Margin := (ClientSize.X - TotalSize.X) div 3
      else
        Margin := (ClientSize.Y - TotalSize.Y) div 3;
      Spacing := Margin;
    end
    else
    begin
      TotalSize := Point(GlyphSize.X + Spacing + TextSize.X, GlyphSize.Y +
        Spacing + TextSize.Y);
      if Layout in [blGlyphLeft, blGlyphRight] then
        Margin := (ClientSize.X - TotalSize.X + 1) div 2
      else
        Margin := (ClientSize.Y - TotalSize.Y + 1) div 2;
    end;
  end
  else
  begin
    if Spacing = -1 then
    begin
      TotalSize := Point(ClientSize.X - (Margin + GlyphSize.X), ClientSize.Y -
        (Margin + GlyphSize.Y));
      if Layout in [blGlyphLeft, blGlyphRight] then
        Spacing := (TotalSize.X - TextSize.X) div 2
      else
        Spacing := (TotalSize.Y - TextSize.Y) div 2;
    end;
  end;
    
  case Layout of
    blGlyphLeft:
      begin
        GlyphPos.X := Margin;
        TextPos.X := GlyphPos.X + GlyphSize.X + Spacing;
      end;
    blGlyphRight:
      begin
        GlyphPos.X := ClientSize.X - Margin - GlyphSize.X;
        TextPos.X := GlyphPos.X - Spacing - TextSize.X;
      end;
    blGlyphTop:
      begin
        GlyphPos.Y := Margin;
        TextPos.Y := GlyphPos.Y + GlyphSize.Y + Spacing;
      end;
    blGlyphBottom:
      begin
        GlyphPos.Y := ClientSize.Y - Margin - GlyphSize.Y;
        TextPos.Y := GlyphPos.Y - Spacing - TextSize.Y;
      end;
  end;
    
  { fixup the result variables }
  with GlyphPos do
  begin
    Inc(X, Client.Left + Offset.X);
    Inc(Y, Client.Top + Offset.Y);
  end;
  OffsetRect(TextBounds, TextPos.X + Client.Left + Offset.X,
    TextPos.Y + Client.Top + Offset.X);
end;

function TXPSpeedButton.MapColor(Color: TColor): TColor;
var
  Index: Byte;
begin
  if (Color = Glyph.TransparentColor) or (ColorToRGB(Color) =
    ColorToRGB(clBtnFace)) then Result := Color
  else begin
    Color := ColorToRGB(Color);
    Index := Byte(Longint(Word(GetRValue(Color)) * 77 +
      Word(GetGValue(Color)) * 150 + Word(GetBValue(Color)) * 29) shr 8);
    Result := RGB(Index, Index, Index);
  end;
end;

procedure TXPSpeedButton.DrawButtonGlyph(Canvas: TCanvas; const GlyphPos: TPoint;
  State: TButtonState; Transparent: Boolean);
var
   X, Y: Integer;
begin
  if (Glyph = nil) or Glyph.Empty then exit;

  Glyph.Transparent:= Transparent;
  if (csDesigning in ComponentState) then
  begin
   if not Enabled then
   begin
    DisabledBitmap:= CreateDisabledBitmap(Glyph, clBlack);
    Canvas.Draw(GlyphPos.X, GlyphPos.Y, DisabledBitmap);
   end
   else Canvas.Draw(GlyphPos.X, GlyphPos.Y, Glyph);
   exit;
  end;

  if not Enabled then
  begin
   DisabledBitmap:= CreateDisabledBitmap(Glyph, clBlack);
   Canvas.Draw(GlyphPos.X, GlyphPos.Y, DisabledBitmap);
  end
  else if not FFocused and FGrayInactive then
       begin
        GrayBitmap.Assign(Glyph);
        with GrayBitmap do
        begin
         for X := 0 to Width - 1 do
             for Y := 0 to Height - 1 do
                 Canvas.Pixels[X, Y] := MapColor(GrayBitmap.Canvas.Pixels[X, Y]);
        end;
        Canvas.Draw(GlyphPos.X, GlyphPos.Y, GrayBitmap)
       end
       else Canvas.Draw(GlyphPos.X, GlyphPos.Y, Glyph);
end;

procedure TXPSpeedButton.DrawButtonText(Canvas: TCanvas; const Caption: string;
  TextBounds: TRect; State: TButtonState; BiDiFlags: LongInt);
begin
  with Canvas do
  begin
    Brush.Style := bsClear;
    if State = bsDisabled then
    begin
      OffsetRect(TextBounds, 1, 1);
      Font.Color := clBtnHighlight;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
        DT_CENTER or DT_VCENTER or BiDiFlags);
      OffsetRect(TextBounds, -1, -1);
      Font.Color := clBtnShadow;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
        DT_CENTER or DT_VCENTER or BiDiFlags);
    end else
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
        DT_CENTER or DT_VCENTER or BiDiFlags);
  end;
end;

procedure Register;
begin
  RegisterComponents('EloSoft', [TXPSpeedButton]);
end;

end.
